import wx
from SimulatorFramework import *


N  = North      = 0
NE = Northeast  = 1
E  = East       = 2
SE = Southeast  = 3
S  = South      = 4
SW = Southwest  = 5
W  = West       = 6
NW = Northwest  = 7
DirList = [N,NE,E,SE,S,SW,W,NW]

DirAbbrevArray = ("N", "NE", "E", "SE", "S", "SW", "W", "NW")
DirNameArray = ("NORTH", "NORTHEAST", "EAST", "SOUTHEAST", "SOUTH", "SOUTHWEST", "WEST", "NORTHWEST")

def GetFontSizeToFitTextInBounds(text, font, dc, width, height):
    retFont = font
    (w, pixelSize) = retFont.GetPixelSize()
    bContinue = True
    while bContinue == True:
        dc.SetFont(retFont)
        (w, h) = dc.GetTextExtent(text)
        if ((w <= width) and (h <= height)) or pixelSize <= 2:
            bContinue = False
        else:
            pixelSize = pixelSize - 2
            retFont.SetPixelSize((0, pixelSize))
    #(w, pixelSize) = retFont.GetPixelSize()
    #print("pixelSize: " + str(pixelSize))
    return retFont
    
class Glue:
    """A representation of a glue on the side of a tile, including strength and label"""
    strength = 0
    label = ""

    def __init__(self, strength = 0, label = ""):
        self.strength = strength
        self.label = label

    def __eq__(self, other):
        if (isinstance(other, Glue)):
            return (self.strength == other.strength) and (self.label == other.label)
        return False

    def Copy(self):
        return Glue(self.strength, self.label)

    def Contains(self, string):
        return (self.label.find(string) > -1)

    def Print(self, d):
        print(DirNameArray[d] + "BIND " + str(self.strength))
        print(DirNameArray[d] + "LABEL " + self.label)

    def Output(self, d):
        ret  = DirNameArray[d] + "BIND " + str(self.strength) + "\n"
        ret += DirNameArray[d] + "LABEL " + self.label + "\n"
        return ret
    
    def Hash(self):
        return self.label + " " + str(self.strength)


CELLSIZE = 60
CELLSIZE = 60

# for 3D simulation
#----------------------------------------------------------
# in OpenGL 1.0 texture sides should be 2^n
TEXTURE_RES = 1024

# size of a side of a tile on the texture
SIDE_RES = int(TEXTURE_RES / 3)

# texture offsets of given side [N,E,S,W,U,D]
SIDE_OFFSET = [(0,0),(0,SIDE_RES),(SIDE_RES,0),(SIDE_RES,SIDE_RES),(2*SIDE_RES,0),(2*SIDE_RES,SIDE_RES)]
#----------------------------------------------------------

class OctTASTileType(TileType):
    """A representation of a tile type for use in the ISU TAS simulator.
       Every tile must have a unique name and a unique set of 4 glues. If
       a name is not explicitly provided, one will be formed automatically
       as a concatenation of the glues values of each side."""

    name = ""
    label = ""
    color = ""
    colorVector = (255, 255, 255)
    sides = {}
    dims  = 2
    imgFile = None
    img   = None
    markingsStr = None

    def __init__(self, name="", width=CELLSIZE, height=CELLSIZE, is3D=False, imgFile=None):
        TileType.__init__(self, name, width, height)
        if not imgFile is None:
            imgFile = imgFile
        if is3D == False:
            self.dims = 2
            self.sides = {N:Glue(),NE:Glue(),E:Glue(),SE:Glue(),S:Glue(),SW:Glue(),W:Glue(),NW:Glue()}
        else:
            print("ERROR: Handling for 3D is not implemented!")
            
    def DrawAtLocation(self, xPixel, yPixel, pdc):
        if self.img is None:
            try:
                self.MakeImage()
            except:
                print("Exception in DrawAtLocation for tile of type " + self.GetName())
                return
        pdc.DrawBitmap(self.img, xPixel, yPixel)

    def GetMarkingsStr(self):
        if self.markingsStr is None:
            self.markingsStr = ""
            for d in range(len(DirAbbrevArray)):
                self.markingsStr = self.markingsStr + DirAbbrevArray[d] + ":" + self.sides[d].label  + "(" + str(self.sides[d].strength) + ") "
        return self.markingsStr

    def MakeImage(self, backgroundColor=None, bForce=False):
        if (not self.img is None) and (bForce == False):
            return
        if self.imgFile is None:
            w = CELLSIZE*3
            h = CELLSIZE*3
            bmp = wx.Bitmap(w, h)
            dc = wx.MemoryDC()
            dc.SelectObject(bmp)
            
            if not backgroundColor is None:
                bg = wx.Brush(backgroundColor)#(self.GetBackgroundColour())
                dc.SetBackground(bg)
            dc.Clear()

            color = wx.TheColourDatabase.Find(self.color)
            brush = wx.Brush(color)
            dc.SetBrush(brush)
            
            tabSize = 10
            dc.DrawRectangle(0+tabSize, 0+tabSize, w-(2*tabSize), h-(2*tabSize))
            
            color = wx.TheColourDatabase.Find("Black")
            brush = wx.Brush(color)
            dc.SetBrush(brush)

            # Draw northern tabs
            numTabs = self.sides[N].strength
            if numTabs > 0:
                spacing = w / (numTabs+1)
                currLoc = spacing
                for n in range(numTabs):
                    dc.DrawRectangle(currLoc-(tabSize/2), 0, tabSize, tabSize)
                    currLoc = currLoc + spacing

            # Draw southern tabs
            numTabs = self.sides[S].strength
            if numTabs > 0:
                spacing = w / (numTabs+1)
                currLoc = spacing
                for n in range(numTabs):
                    dc.DrawRectangle(currLoc-(tabSize/2), h-tabSize, tabSize, tabSize)
                    currLoc = currLoc + spacing
            
            # Draw western tabs
            numTabs = self.sides[W].strength
            if numTabs > 0:
                spacing = h / (numTabs+1)
                currLoc = spacing
                for n in range(numTabs):
                    dc.DrawRectangle(0, currLoc-(tabSize/2), tabSize, tabSize)
                    currLoc = currLoc + spacing

            # Draw eastern tabs
            numTabs = self.sides[E].strength
            if numTabs > 0:
                spacing = h / (numTabs+1)
                currLoc = spacing
                for n in range(numTabs):
                    dc.DrawRectangle(w-tabSize, currLoc-(tabSize/2), tabSize, tabSize)
                    currLoc = currLoc + spacing

            # Draw tile label
            wLabel = (w - 2*tabSize)/2
            hLabel = (h - 2*tabSize)/2
            defPixHt = 100
            ftype = wx.FONTFAMILY_MODERN
            glueLabelScale = 0.6
            
            fname = "Arial Narrow"
            #font = wx.Font(pointSize = 30, family = wx.DEFAULT, style = wx.NORMAL, weight = wx.NORMAL, faceName = 'Arial')
            font = wx.Font(pixelSize = (0,defPixHt), family = ftype, style = wx.NORMAL, weight = wx.FONTWEIGHT_BOLD, faceName = fname)#, faceName = 'Arial')
            font = GetFontSizeToFitTextInBounds(self.label, font, dc, wLabel, hLabel)
            (wLabel, hLabel) = dc.GetTextExtent(self.label)
            dc.DrawText(self.label, (w/2)-(wLabel/2), (h/2)-(hLabel/2))
            
            # Draw north glue label
            wLabel = ((w - 2*tabSize)/2) * glueLabelScale
            hLabel = ((h - 2*tabSize)/2) * glueLabelScale
            font = wx.Font(pixelSize = (0,defPixHt), family = ftype, style = wx.NORMAL, weight = wx.FONTWEIGHT_NORMAL, faceName = fname)
            font = GetFontSizeToFitTextInBounds(self.sides[N].label, font, dc, wLabel, hLabel)
            (wLabel, hLabel) = dc.GetTextExtent(self.sides[N].label)
            dc.DrawText(self.sides[N].label, (w/2)-(wLabel/2), (h/4)-(hLabel/2))
            
            # Draw south glue label
            wLabel = ((w - 2*tabSize)/2) * glueLabelScale
            hLabel = ((h - 2*tabSize)/2) * glueLabelScale
            font = wx.Font(pixelSize = (0,defPixHt), family = ftype, style = wx.NORMAL, weight = wx.FONTWEIGHT_NORMAL, faceName = fname)
            font = GetFontSizeToFitTextInBounds(self.sides[S].label, font, dc, wLabel, hLabel)
            (wLabel, hLabel) = dc.GetTextExtent(self.sides[S].label)
            dc.DrawText(self.sides[S].label, (w/2)-(wLabel/2), (3*h/4)-(hLabel/2))
            
            # Draw west glue label
            wLabel = ((w - 2*tabSize)/2) * glueLabelScale
            hLabel = ((h - 2*tabSize)/2) * glueLabelScale
            font = wx.Font(pixelSize = (0,defPixHt), family = ftype, style = wx.NORMAL, weight = wx.FONTWEIGHT_NORMAL, faceName = fname)
            font = GetFontSizeToFitTextInBounds(self.sides[W].label, font, dc, wLabel, hLabel)
            (wLabel, hLabel) = dc.GetTextExtent(self.sides[W].label)
            dc.DrawRotatedText(self.sides[W].label, (w/4)-1.5*(hLabel/2), (h/2)+(wLabel/2), 90.0)

            # Draw east glue label
            wLabel = ((w - 2*tabSize)/2) * glueLabelScale
            hLabel = ((h - 2*tabSize)/2) * glueLabelScale
            font = wx.Font(pixelSize = (0,defPixHt), family = ftype, style = wx.NORMAL, weight = wx.FONTWEIGHT_NORMAL, faceName = fname)
            font = GetFontSizeToFitTextInBounds(self.sides[E].label, font, dc, wLabel, hLabel)
            (wLabel, hLabel) = dc.GetTextExtent(self.sides[E].label)
            dc.DrawRotatedText(self.sides[E].label, (3*w/4)+1.5*(hLabel/2), (h/2)-(wLabel/2), 270.0)

            dc.SelectObject(wx.NullBitmap)
            
            img = bmp.ConvertToImage()
            img = img.Scale(CELLSIZE, CELLSIZE, wx.IMAGE_QUALITY_HIGH)
            bmp = wx.Bitmap(img)
            self.img = bmp
        else:
            try:
                img = wx.Image(self.imgFile, wx.BITMAP_TYPE_TIF).ConvertToBitmap()
            except:
                print("Exception thrown trying to load image file '" + self.imgFile + "'")
                img = None
            if (img is None) or (img.Width > 1000):
                bmp = wx.Bitmap(self.width-20, self.height-20)
                dc = wx.MemoryDC()
                dc.SelectObject(bmp)
                dc.Clear()
                colorStr = "Red"
                if img is None:
                    colorStr = "Blue"
                else:
                    #globalApp.OutputMessage("WARNING:  Invalid tile image file '" + imgFile + "'")
                    print("WARNING:  Invalid tile image file '" + self.imgFile + "'")
                color = wx.TheColourDatabase.Find(colorStr)
                brush = wx.Brush(color)
                dc.SetBrush(brush)
                dc.DrawCircle(self.width/2, self.height/2, (self.width-40)/2)
                dc.SelectObject(wx.NullBitmap)
                img = bmp.ConvertToImage()
            else:
                try:
                    img = img.ConvertToImage()
                    #img = img.Scale(self.sideLen, self.sideLen, wx.IMAGE_QUALITY_HIGH)
                    img = img.Scale(self.width+40, self.height+40, wx.IMAGE_QUALITY_HIGH)
                    #img = img.Rotate(math.radians(90), (48, 48))
                except:
                    print("Failed to create tile image from file " + self.imgFile)
                    self.img = None
                    return
            img = wx.Bitmap(img)
            self.img = img

    def Copy(self):
        t = TileType()
        for d in DirList:
            t.sides[d] = self.sides[d].Copy()
        t.name = self.name
        t.label = self.label
        t.color = self.color
        t.colorVector = self.colorVector
        return t

    def Print(self):
        if self.IsEmpty():
            return
        if self.name == "":
            self.CreateName()
        print("TILENAME " + self.name)
        print("LABEL " + self.label)
        for i,d in self.sides.items():
            self.sides[i].Print(i)
        if len(self.color) > 0:
            print("TILECOLOR " + self.color)
        print("CREATE\n")

    def Output(self):
        if self.IsEmpty():
            return ""
        if self.name == "":
            self.CreateName()
        ret  = "TILENAME " + self.name + "\n"
        ret += "LABEL " + self.label + "\n"
        for i,d in self.sides.items():
            ret += self.sides[i].Output(i)
        if len(self.color) > 0:
            ret += "TILECOLOR " + self.color + "\n"
        ret += "CREATE\n"
        return ret

    def SetInputSides(self, inputSet):
        for k in inputSet:
            self.sides[k] = Glue(inputSet[k].strength, inputSet[k].label)

    def IsEmpty(self):
        for i in range(8):
            if not self.sides[i].strength == 0:
                return False
        return True

    def GetName(self):
        if len(self.name) == 0:
            self.CreateName(self)
        return self.name

    def CreateName(self):
        for i in range(8):
            self.name = self.name + DirAbbrevArray[i] + str(self.sides[i].strength) + self.sides[i].label

    # function to parse the color string into a color vector
    def ParseColor(self):
        col = wx.TheColourDatabase.Find(self.color)
        if col.IsOk():
            self.colorVector = (col.Red(), col.Green(), col.Blue())
        else:
            if not self.color.startswith("rgb"):
                self.colorVector = (255, 255, 255)
            else:
                channels = self.color[4:-1].split(",")
                r = int(channels[0].strip())
                g = int(channels[1].strip())
                b = int(channels[2].strip())
                self.colorVector = (r, g, b)

